@@ -44,12 +44,15 @@ class LinearOperator(spLinearOperator):
         (``True``) or not (``False``)
 
     """
-    def __init__(self, Op=None, explicit=False):
+    def __init__(self, Op=None, explicit=False, clinear=None):
         self.explicit = explicit
         if Op is not None:
             self.Op = Op
             self.shape = self.Op.shape
             self.dtype = self.Op.dtype
+            self.clinear = getattr(self.Op, 'clinear', True)
+        if clinear is not None:
+            self.clinear = clinear
 
     def _matvec(self, x):
         if callable(self.Op._matvec):
@@ -71,8 +74,6 @@ class LinearOperator(spLinearOperator):
             y = np.vstack([self.matvec(col.reshape(-1)) for col in X.T]).T
         return y
 
-        return np.vstack([self.matvec(col.reshape(-1)) for col in X.T]).T
-
     def __mul__(self, x):
         y = super().__mul__(x)
         if isinstance(y, spLinearOperator):
@@ -240,11 +241,15 @@ class LinearOperator(spLinearOperator):
 
         """
         if isinstance(x, LinearOperator):
-            return _ProductLinearOperator(self, x)
+            Op = _ProductLinearOperator(self, x)
+            # Output is C-Linear only if both operators are
+            Op.clinear = (getattr(self, 'clinear', True)
+                          and getattr(x, 'clinear', True))
+            return Op
         elif np.isscalar(x):
             return _ScaledLinearOperator(self, x)
         else:
-            if x.ndim == 1 or x.ndim == 2 and x.shape[1] == 1:
+            if x.ndim == 1:# or x.ndim == 2 and x.shape[1] == 1:
                 return self.matvec(x)
             elif x.ndim == 2:
                 return self.matmat(x)
@@ -397,7 +402,7 @@ class LinearOperator(spLinearOperator):
         # post process the entries / indices for scipy.sparse.csr_matrix
         entries = np.array(entries)
         indices = np.array(indices)
-        i, j = indices[:,0], indices[:,1]
+        i, j = indices[:, 0], indices[:, 1]
 
         # construct a sparse, CSR matrix from the entries / indices data.
         matrix = csr_matrix((entries, (j, i)), shape=self.shape, dtype=self.dtype)
@@ -482,7 +487,7 @@ class LinearOperator(spLinearOperator):
                                          'for non-symmetric square matrices of '
                                          'complex type...')
                     if symmetric and uselobpcg:
-                        X = np.random.rand(self.shape[0], neigs)
+                        X = np.random.rand(self.shape[0], neigs).astype(self.dtype)
                         eigenvalues = \
                             sp_lobpcg(self.A, X=X, maxiter=niter,
                                       **kwargs_eig)[0]
@@ -499,7 +504,7 @@ class LinearOperator(spLinearOperator):
                                                          self.A)))
                 else:
                     if uselobpcg:
-                        X = np.random.rand(self.shape[1], neigs)
+                        X = np.random.rand(self.shape[1], neigs).astype(self.dtype)
                         eigenvalues = np.sqrt(sp_lobpcg(
                             np.dot(np.conj(self.A.T), self.A),
                             X=X, maxiter=niter, **kwargs_eig)[0])
@@ -516,7 +521,7 @@ class LinearOperator(spLinearOperator):
                                      'non symmetric square matrices of '
                                      'complex type...')
                 if symmetric and uselobpcg:
-                    X = np.random.rand(self.shape[0], neigs)
+                    X = np.random.rand(self.shape[0], neigs).astype(self.dtype)
                     eigenvalues = \
                         sp_lobpcg(self, X=X, maxiter=niter, **kwargs_eig)[0]
                 elif symmetric:
@@ -527,7 +532,7 @@ class LinearOperator(spLinearOperator):
                                           **kwargs_eig)[0]
             else:
                 if uselobpcg:
-                    X = np.random.rand(self.shape[1], neigs)
+                    X = np.random.rand(self.shape[1], neigs).astype(self.dtype)
                     eigenvalues = np.sqrt(sp_lobpcg(self.H * self, X=X,
                                                     maxiter=niter,
                                                     **kwargs_eig)[0])
@@ -628,6 +633,44 @@ class LinearOperator(spLinearOperator):
         colop = _ColumnLinearOperator(self, cols)
         return colop
 
+    def toreal(self, forw=True, adj=True):
+        """Real operator
+
+        Parameters
+        ----------
+        forw : :obj:`bool`, optional
+            Apply real to output of forward pass
+        adj : :obj:`bool`, optional
+            Apply real to output of adjoint pass
+
+        Returns
+        -------
+        realop : :obj:`pylops.LinearOperator`
+            Real operator
+
+        """
+        realop = _RealImagLinearOperator(self, forw, adj, True)
+        return realop
+
+    def toimag(self, forw=True, adj=True):
+        """Imag operator
+
+        Parameters
+        ----------
+        forw : :obj:`bool`, optional
+            Apply imag to output of forward pass
+        adj : :obj:`bool`, optional
+            Apply imag to output of adjoint pass
+
+        Returns
+        -------
+        imagop : :obj:`pylops.LinearOperator`
+            Imag operator
+
+        """
+        imagop = _RealImagLinearOperator(self, forw, adj, False)
+        return imagop
+
 
 def _get_dtype(operators, dtypes=None):
     if dtypes is None:
@@ -729,6 +772,45 @@ class _ColumnLinearOperator(LinearOperator):
         return y
 
 
+class _RealImagLinearOperator(LinearOperator):
+    """Real-Imag linear operator
+
+    Computes forward and adjoint passes of an operator Op and returns only
+    its real (or imaginary) component. Note that for the adjoint step the
+    output must be complex conjugated (i.e. opposite of the imaginary part is
+    returned)
+    """
+    def __init__(self, Op, forw=True, adj=True, real=True):
+        if not isinstance(Op, spLinearOperator):
+            raise TypeError('Op must be a LinearOperator')
+        super(_RealImagLinearOperator, self).__init__(Op, Op.shape)
+        self.Op = Op
+        self.real = real
+        self.forw = forw
+        self.adj = adj
+        self.dtype = np.array(0, dtype=self.Op.dtype).real.dtype
+
+    def _matvec(self, x):
+        ncp = get_array_module(x)
+        y = self.Op._matvec(x)
+        if self.forw:
+            if self.real:
+                y = ncp.real(y)
+            else:
+                y = ncp.imag(y)
+        return y
+
+    def _rmatvec(self, x):
+        ncp = get_array_module(x)
+        y = self.Op._rmatvec(x)
+        if self.adj:
+            if self.real:
+                y = ncp.real(y)
+            else:
+                y = -ncp.imag(y)
+        return y
+
+
 def aslinearoperator(Op):
     """Return Op as a LinearOperator.
 
